--[[
	
	'Show Money' script by Adrian Gajos #18271
	
	Author: Adrian Gajos
	Version: 1.3
	Release: 06.05.2017
	
	
	GNU General Public License:
		
		freedom 0 - The freedom to run the program as you wish, for any purpose.
		freedom 1 - The freedom to study how the program works, and change it so it does your computing as you wish.
		freedom 2 - The freedom to redistribute copies so you can help your neighbor.
		freedom 3 - The freedom to distribute copies of your modified versions to others.

]]


-------------
-- LIBRARY --
-------------
local lib = {}

function lib.initArray(m, v)
	local array = {}
	local err = false
	
	for i = 1, m do
		if type(v) ~= 'table' then
			array[i] = v or nil
		elseif #v == 0 then
			array[i] = {}
		else
			print_error('lib.initArray(' .. m .. ', table): this table contains element(s). Setting \'nil\' for this table.', 'SYNTAX')
			return nil
		end
	end
	
	return array
end

function lib.tableContains(tab, value)
	for i, v in pairs(tab) do
		if v == value then
			return true
		end
	end
	
	return false
end

function print_error(message, prefix)
	local prefix = prefix .. ' ' or ''
	print('\169255000000' .. prefix .. 'ERROR: ' .. message)
end

----------
-- INIT --
----------
sm = {}
sm._DIR = 'sys/lua/autorun'
sm._PLAYERS = {}
sm._TEAM1 = {}
sm._TEAM2 = {}

sm.team = lib.initArray(32, 0)
sm.rifleid = lib.initArray(32, false)
sm.rifle_shadowid = lib.initArray(32, false)
sm.shadowid = lib.initArray(32, false)
sm.bombid = {0, false}

-----------
-- HOOKS --
-----------
addhook('startround', 'sm.initHud', 42)

function sm.onBuy(id)
	timer(50, 'parse', 'lua "sm.updateHud(' .. id .. ')"', 1)
end

function sm.onDrop(id)
	timer(50, 'parse', 'lua "sm.updateIcons(' .. id .. ')"', 1)
end

function sm.onTeam(id)
	sm.container = function()
		if player(id, 'health') > 0 then
			local team = player(id, 'team')
			
			sm.team[id] = team
			table.insert(sm._PLAYERS, id)
			if team == 1 then
				table.insert(sm._TEAM1, id)
			else
				table.insert(sm._TEAM2, id)
			end
			
			sm.updateHud(id)
		end
	end
	
	timer(100, 'sm.container', nil, 1)
end

------------
-- ENGINE --
------------
function sm.initHud()
	local freezeTime = tonumber(game('mp_freezetime')) or 0
	
	if freezeTime > 1 then
		sm._PLAYERS = player(0, 'tableliving')
		sm._TEAM1 = player(0, 'team1living')
		sm._TEAM2 = player(0, 'team2living')
		
		for _, id in pairs(sm._PLAYERS) do
			sm.team[id] = player(id, 'team')
			sm.updateHud(id)
		end
		
		addhook('buy', 'sm.onBuy', 42)
		addhook('drop', 'sm.onDrop', 42)
		addhook('team', 'sm.onTeam', 42)
		timer((freezeTime - 1) * 1000, 'sm.removeHud', nil, 1)
	end
end

function sm.updateHud(id)
	sm.updateMoney(id)
	sm.updateIcons(id)
end

function sm.removeHud()
	freehook('buy','sm.onBuy')
	freehook('drop','sm.onDrop')
	freehook('team','sm.onTeam')
	
	for _, id in pairs(sm._PLAYERS) do
		local teammates = sm.getTeammates(id)
		
		for _, pl in pairs(teammates) do
			hudtxtalphafade(id, pl, 1000, 0)
		end
		
		if sm.rifleid[id] then
			tween_alpha(sm.rifleid[id], 1000, 0)
			tween_alpha(sm.rifle_shadowid[id], 1000, 0)
			timer(1000, 'freeimage', sm.rifleid[id], 1)
			timer(1000, 'freeimage', sm.rifle_shadowid[id], 1)
		end
		
		if sm.shadowid[id] then
			tween_alpha(sm.shadowid[id], 1000, 0)
			timer(1000, 'freeimage', sm.shadowid[id], 1)
			sm.shadowid[id] = false
		end
	end
	
	if sm.bombid[1] > 0 then
		sm.bombid[1] = 0
		tween_alpha(sm.bombid[2], 1000, 0)
		timer(1000, 'freeimage', sm.bombid[2], 1)
		sm.bombid[2] = false
	end
end

function sm.updateMoney(id)
	local teammates = sm.getTeammates(id)
	
	for _, pl in pairs(teammates) do
		local x, y = player(id, 'x') - player(pl, 'x') + 425, player(id, 'y') - player(pl, 'y') + 205
		local money = player(id, 'money')
		local greenRange = (team == 1) and 3150 or 3750
		
		local color = (money >= greenRange) and
			'000255000'
		or (money >= 1000) and
			'255255000'
		or
			'255000000'
		
		hudtxt2(pl, id, '\169' .. color .. '$' .. money, x, y, 1)
	end
end

function sm.updateIcons(id)
	if player(id, 'bomb') and sm.bombid[1] == 0 then
		sm.bombid[1] = id
		sm.bombid[2] = image('gfx/sprites/flare4.bmp<a>', 0, 1, id + 100)
		imagecolor(sm.bombid[2], 255, 0, 0)
		imagealpha(sm.bombid[2], 0.4)
	elseif not player(id, 'bomb') and id == sm.bombid[1] then
		sm.bombid[1] = 0
		freeimage(sm.bombid[2])
		sm.bombid[2] = false
	end
	
	local allowedItems = {10, 11, 20, 21, 22, 23, 24, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 91, 40, 45, 46, 47, 48, 49, 88, 90, 41}
	local itemList = playerweapons(id)
	for i, v in pairs(itemList) do
		if sm.rifleid[id] then
			freeimage(sm.rifleid[id])
			freeimage(sm.rifle_shadowid[id])
			sm.rifleid[id] = false
			sm.rifle_shadowid[id] = false
		end
		
		if lib.tableContains(allowedItems, v) then
			local x, y = player(id, 'x'), player(id, 'y')
			local weaponName = itemtype(v, 'name')
			
			-- shadow of rifle
			sm.rifle_shadowid[id] = image('gfx/shadow.bmp<a>', x, y + 30, 3)
			imagecolor(sm.rifle_shadowid[id], 0, 0, 0)
			imagealpha(sm.rifle_shadowid[id], 0.7)
			
			-- common rifle
			weaponName = weaponName:gsub('-',''):gsub(' ',''):lower()
			sm.rifleid[id] = image('gfx/weapons/' .. weaponName .. '_k.bmp', x, y + 30, 3)
			imagealpha(sm.rifleid[id], 0.8)
			
			return
		end
	end
	
	-- player shadow
	if not sm.shadowid[id] then
		sm.shadowid[id] = image('gfx/sprites/snowflake.bmp<a>', 0, 1, 100 + id)
		imagecolor(sm.shadowid[id], 0, 0, 0)
		imagealpha(sm.shadowid[id], 0.5)
	end
end

function sm.getTeammates(id)
	local team = sm.team[id]
	local teammates = (team == 1) and sm._TEAM1 or sm._TEAM2
	
	return teammates
end

-------------
-- WRAPPER --
-------------
function hudtxt2(playerid, id, text, x, y, align)
	parse('hudtxt2 ' .. playerid .. ' ' .. id .. ' "' .. text .. '" ' .. x .. ' ' .. y .. ' ' .. align)
end

function hudtxtalphafade(playerid, id, duration, alpha)
	parse('hudtxtalphafade ' .. playerid .. ' ' .. id .. ' ' .. duration .. ' ' .. alpha)
end